part of 'app_route.dart';

class ActivityRoute<T> extends AppRoute<T> {
  final RouteKeyBuilder? key;
  final RouteTitleBuilder? title;
  final String? restorationId;
  final bool maintainState;
  final bool fullscreenDialog;
  final SystemUiStyleBuilder systemUiStyle;
  final RouteWidgetBuilder builder;

  const ActivityRoute({
    this.key,
    this.title,
    this.restorationId,
    required String location,
    this.maintainState = true,
    this.fullscreenDialog = false,
    this.systemUiStyle = defaultSystemUiStyle,
    required this.builder,
  }) : super._(location: location);

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) =>
      AppRouterDelegate.of(context).push(this, args: args);

  @factory
  AppPage<T> createPage({RouteArgs args = const {}}) => AppPage._(route: this, args: args);
}

class AppPage<T> extends Page {
  final Completer<T> _result = Completer();

  final ActivityRoute<T> route;

  Future<T> get result => _result.future;

  bool get finished => _result.isCompleted;

  final List<String?> _location = [null];

  String get location => _location[0] ?? route.location;

  set location(String? value) => _location[0] = value;

  @override
  RouteArgs get arguments => super.arguments as RouteArgs;

  AppPage._({required this.route, required RouteArgs args})
      : super(
            key: route.key?.call(args),
            name: route.title?.call(args),
            arguments: args,
            restorationId: route.restorationId);

  @override
  Route<T> createRoute(BuildContext context) => MaterialPageRoute(
      builder: (context) =>
          AnnotatedSystemUIStyle(style: route.systemUiStyle(context), child: route.builder(context, arguments)),
      settings: this,
      maintainState: route.maintainState,
      fullscreenDialog: route.fullscreenDialog)
    ..popped.then(_result.complete, onError: _result.completeError);

  @override
  String toString() => '$runtimeType(${route.location})';

  @optionalTypeArgs
  static AppPage<T> of<T>(BuildContext context) => AppRouterDelegate.of(context).currentConfiguration as AppPage<T>;
}
