part of 'app_route.dart';

class BottomSheetRoute<T> extends AppRoute<T> {
  final Color? backgroundColor;
  final double? elevation;
  final ShapeBorder? shape;
  final Clip? clipBehavior;
  final BoxConstraints? constraints;
  final Color? barrierColor;
  final bool isScrollControlled;
  final bool useRootNavigator;
  final bool isDismissible;
  final bool enableDrag;
  final bool routeSettings;
  final AnimationController? transitionAnimationController;
  final SystemUiStyleBuilder systemUiStyle;
  final RouteWidgetBuilder builder;

  const BottomSheetRoute({
    required String location,
    this.backgroundColor,
    this.elevation,
    this.shape,
    this.clipBehavior,
    this.constraints,
    this.barrierColor,
    this.isScrollControlled = false,
    this.useRootNavigator = false,
    this.isDismissible = true,
    this.enableDrag = true,
    this.routeSettings = true,
    this.transitionAnimationController,
    this.systemUiStyle = defaultSystemUiStyle,
    required this.builder,
  }) : super._(location: location);

  BottomSheetRoute.draggable({
    double initialChildSize = .6,
    double minChildSize = .2,
    double maxChildSize = 1,
    double? contentHeight,
    bool updateInsets = true,
    required String location,
    Color? backgroundColor,
    double? elevation,
    ShapeBorder? shape,
    Clip? clipBehavior,
    this.barrierColor,
    this.useRootNavigator = false,
    this.isDismissible = true,
    this.routeSettings = false,
    this.transitionAnimationController,
    this.systemUiStyle = defaultSystemUiStyle,
    required RouteWidgetBuilder builder,
  })  : backgroundColor = Colors.transparent,
        elevation = 0,
        shape = null,
        clipBehavior = Clip.none,
        constraints = null,
        isScrollControlled = true,
        enableDrag = true,
        builder = ((context, arguments) => FullscreenDraggableBottomSheet(
            initialChildSize: initialChildSize,
            minChildSize: minChildSize,
            maxChildSize: maxChildSize,
            contentHeight: contentHeight,
            backgroundColor: backgroundColor,
            elevation: elevation,
            shape: shape,
            clipBehavior: clipBehavior,
            updateInsets: updateInsets,
            child: builder(context, arguments))),
        super._(location: location);

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) => showModalBottomSheet<T>(
          context: context,
          backgroundColor: backgroundColor,
          elevation: elevation,
          shape: shape,
          clipBehavior: clipBehavior,
          constraints: constraints,
          barrierColor: barrierColor,
          isScrollControlled: isScrollControlled,
          useRootNavigator: useRootNavigator,
          isDismissible: isDismissible,
          enableDrag: enableDrag,
          routeSettings: routeSettings ? RouteSettings(name: location, arguments: args) : null,
          transitionAnimationController: transitionAnimationController,
          builder: (context) => AnnotatedSystemUIStyle(style: systemUiStyle(context), child: builder(context, args)))
      as Future<T>;
}
